from __future__ import annotations

import argparse
import os
from datetime import datetime

from zett.constants import ZETTELKASTEN_SKELETON


def add_subparser(
    subparsers: argparse._SubParsersAction,  # type: ignore
    parents: list[argparse.ArgumentParser],
) -> None:
    parser = subparsers.add_parser(
        "generate",
        parents=parents,
        help="Generate a new zettel",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "title",
        help="The title of the zettel",
    )
    parser.add_argument(
        "--tags",
        help="A list of comma separated tags",
    )

    parser.set_defaults(func=run)


def run(args: argparse.Namespace) -> int:
    zettel_date, pretty_date = _get_formatted_dates()
    tags = ""
    if args.tags:
        if "," not in args.tags:
            print("Use a comma to separate the tags.")
            return 1

        tags = _format_tags(tags=args.tags)

    zettel = ZETTELKASTEN_SKELETON.format(
        title=args.title,
        date=pretty_date,
        tags=tags,
    )

    zettel_file = os.path.join(args.directory, f"{zettel_date}.md")

    try:
        with open(zettel_file, mode="w", encoding="utf-8") as fd:
            fd.write(zettel)
    except FileNotFoundError:
        print(f"Failed to create zettel at {zettel_file}")
        return 1

    print(f"Zettel generated at {zettel_file}")
    return 0


def _format_tags(tags: str) -> str:
    splitted_tags = tags.split(",")

    formatted_tags = "tags:"
    for tag in splitted_tags:
        formatted_tags += f"\n   - {tag}"

    formatted_tags += "\n"
    return formatted_tags


def _get_formatted_dates() -> tuple[str, str]:
    now = datetime.now()
    first_date = now.strftime("%Y%m%d%H%M%S")
    second_date = now.strftime("%Y-%m-%d %H:%M:%S %z")

    return first_date, second_date
